@@ -1,7 +1,13 @@
-import torch
 import math
+import torch
 from torch import nn
 import torch.nn.functional as F
+from einops import rearrange
+
+# helpers
+
+def exists(val):
+    return val is not None
 
 # sin activation
 
@@ -34,7 +40,7 @@ class Siren(nn.Module):
         w_std = (1 / dim) if self.is_first else (math.sqrt(c / dim) / w0)
         weight.uniform_(-w_std, w_std)
 
-        if bias is not None:
+        if exists(bias):
             bias.uniform_(-w_std, w_std)
 
     def forward(self, x):
@@ -62,8 +68,32 @@ class SirenNet(nn.Module):
             ))
 
         self.net = nn.Sequential(*layers)
+
+        final_activation = nn.Identity() if not exists(final_activation) else final_activation
         self.last_layer = Siren(dim_in = dim_hidden, dim_out = dim_out, w0 = w0, use_bias = use_bias, activation = final_activation)
 
     def forward(self, x):
         x = self.net(x)
         return self.last_layer(x)
+
+# wrapper
+
+class SirenWrapper(nn.Module):
+    def __init__(self, net, image_width, image_height):
+        super().__init__()
+        assert isinstance(net, SirenNet), 'SirenWrapper must receive a Siren network'
+
+        self.net = net
+        self.image_width = image_width
+        self.image_height = image_height
+
+        tensors = [torch.linspace(-1, 1, steps = image_width), torch.linspace(-1, 1, steps = image_height)]
+        mgrid = torch.stack(torch.meshgrid(*tensors), dim=-1)
+        mgrid = rearrange(mgrid, 'h w c -> (h w) c')
+        self.register_buffer('grid', mgrid)
+
+    def forward(self):
+        coords = self.grid.clone().detach().requires_grad_()
+        out = self.net(coords)
+        out = rearrange(out, '(h w) c -> () c h w', h = self.image_height, w = self.image_width)
+        return out
